/* This file defines the structure of the database.

   Copyright (c) 2007-2022, Weird Solutions Inc. All rights reserved.
*/

/* Domain definitions */

CREATE DOMAIN BOOL AS SMALLINT
DEFAULT 0
CHECK (VALUE IN (0,1));

CREATE DOMAIN BOOT_PROTO_TYPE AS VARCHAR (20) CHARACTER SET ISO8859_1
DEFAULT 'dhcpv4'
CHECK (VALUE IN ('bootp', 'dhcpv4', 'dhcpv6'));

CREATE DOMAIN DHCP_OPTION_TYPE AS VARCHAR (40) CHARACTER SET ISO8859_1
DEFAULT 'unknown'
CHECK (VALUE IN ('8bit','16bit','24bit','32bit','64bit','128bit','string','ipaddress','boolean','time','byte_sequence','topencoded','subencoded','dnsname','expression','control','fixed_offset','snmp_oid','unknown'));

CREATE DOMAIN TID_CLASS AS INTEGER
DEFAULT 0
CHECK (VALUE IN (0,1,2,3,4));

CREATE DOMAIN ENCODING_TYPE AS INTEGER
DEFAULT 0;

CREATE DOMAIN FOREIGN_KEY_ID AS INTEGER
DEFAULT -1;

CREATE DOMAIN INT32_T AS INTEGER
DEFAULT -1;

CREATE DOMAIN AMOUNT_TYPE AS INTEGER
DEFAULT 0;

CREATE DOMAIN DOM_ID_TYPE AS INTEGER
DEFAULT 0;

CREATE DOMAIN ID_CLASS_TYPE AS VARCHAR (20) CHARACTER SET ISO8859_1
CHECK (VALUE IN ('login','device4','device6'));

CREATE DOMAIN ACCESS_ID_TYPE AS INTEGER
DEFAULT 0;

CREATE DOMAIN RIGHTS_TYPE AS INTEGER
DEFAULT 0;

CREATE DOMAIN INDEX_TYPE AS INTEGER
DEFAULT -1;

CREATE DOMAIN TYPE_AS_INT AS INTEGER
DEFAULT -1;

CREATE DOMAIN ID_AS_INT AS INTEGER
DEFAULT -1;

CREATE DOMAIN IP_ADDRESS AS VARCHAR (80) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN TIME_T AS NUMERIC (8,0)
DEFAULT 0;

CREATE DOMAIN EXPRESSION_TYPE AS VARCHAR (1024) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN SELECT_CRITERIA AS VARCHAR (1024) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN CLASS_NAME AS VARCHAR (80) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN LONG_TEXT AS BLOB sub_type 1 segment size 1;

CREATE DOMAIN GENERAL_NAME AS VARCHAR (80) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN HEXSTORE AS VARCHAR (1024) CHARACTER SET ISO8859_1
DEFAULT '';

CREATE DOMAIN NAME_LIST VARCHAR (1024) CHARACTER SET ISO8859_1
DEFAULT '';

/* A table for holding address ranges */

CREATE TABLE DHCP_ADDRESS_RANGE (
    PK INTEGER NOT NULL,
    START IP_ADDRESS COLLATE ISO8859_1,
    STOP IP_ADDRESS COLLATE ISO8859_1);

/* Holds excluded address ranges. */
CREATE TABLE DHCP_EXC_ADDR_RANGE (
    PK INTEGER NOT NULL,
    START IP_ADDRESS COLLATE ISO8859_1,
    STOP IP_ADDRESS COLLATE ISO8859_1);

/* And a simple IP address */
CREATE TABLE WSI_ADDRESS (
    PK INTEGER NOT NULL,
    SRC_PK INTEGER,
    SRC_TABLE GENERAL_NAME COLLATE ISO8859_1,
    ADDR IP_ADDRESS COLLATE ISO8859_1);

/* A table for holding attributes for option elements, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_ELEMENT_ATTR4 (
    PK INTEGER NOT NULL,
    SIGNED BOOL,
    ARRAYED BOOL,
    NULL_TERMINATED BOOL,
    LEN_PREFIX_WIDTH AMOUNT_TYPE,
    SUBTAG_WIDTH AMOUNT_TYPE,
    SUBLEN_WIDTH AMOUNT_TYPE,
    SUBTYPE_WIDTH AMOUNT_TYPE,
    UNITS GENERAL_NAME COLLATE ISO8859_1,
    MIN_VALUE INT32_T,
    MAX_VALUE INT32_T);

CREATE TABLE DHCP_ELEMENT_ATTR6 (
    PK INTEGER NOT NULL,
    SIGNED BOOL,
    ARRAYED BOOL,
    NULL_TERMINATED BOOL,
    LEN_PREFIX_WIDTH AMOUNT_TYPE,
    SUBTAG_WIDTH AMOUNT_TYPE,
    SUBLEN_WIDTH AMOUNT_TYPE,
    SUBTYPE_WIDTH AMOUNT_TYPE,
    UNITS GENERAL_NAME COLLATE ISO8859_1,
    MIN_VALUE INT32_T,
    MAX_VALUE INT32_T);

/* A table for holding attributes for options, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_OPTION_ATTR4 (
    PK INTEGER NOT NULL,
    TAGPATH GENERAL_NAME COLLATE ISO8859_1,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    CLASS GENERAL_NAME COLLATE ISO8859_1,
    DATA_TYPE DHCP_OPTION_TYPE COLLATE ISO8859_1,
    INPUT_ENCODING ENCODING_TYPE,
    OUTPUT_ENCODING ENCODING_TYPE,
    USER_DEFINABLE GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    MAX_INSTANCES AMOUNT_TYPE,
    VENDOR_ID GENERAL_NAME COLLATE ISO8859_1,
    VENDOR_ORO BOOL,
    CONTEXT_VENDORID GENERAL_NAME COLLATE ISO8859_1,
    DEFAULT_VALUE GENERAL_NAME COLLATE ISO8859_1,
    FIXED_OFFSETS NAME_LIST COLLATE ISO8859_1,
    ELEMENT_ATTR_REF FOREIGN_KEY_ID,
    ELEMENT_ATTR_REF_CL CLASS_NAME,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_OPTION_ATTR6 (
    PK INTEGER NOT NULL,
    TAGPATH GENERAL_NAME COLLATE ISO8859_1,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    CLASS GENERAL_NAME COLLATE ISO8859_1,
    DATA_TYPE DHCP_OPTION_TYPE COLLATE ISO8859_1,
    INPUT_ENCODING ENCODING_TYPE,
    OUTPUT_ENCODING ENCODING_TYPE,
    USER_DEFINABLE GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    MAX_INSTANCES AMOUNT_TYPE,
    VENDOR_ID GENERAL_NAME COLLATE ISO8859_1,
    VENDOR_ORO BOOL,
    CONTEXT_VENDORID GENERAL_NAME COLLATE ISO8859_1,
    DEFAULT_VALUE GENERAL_NAME COLLATE ISO8859_1,
    FIXED_OFFSETS NAME_LIST COLLATE ISO8859_1,
    ELEMENT_ATTR_REF FOREIGN_KEY_ID,
    ELEMENT_ATTR_REF_CL CLASS_NAME,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding attributes for vendors, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_VENDOR_ATTR4 (
    PK INTEGER NOT NULL,
    VID GENERAL_NAME COLLATE ISO8859_1,
    VCLASS GENERAL_NAME COLLATE ISO8859_1,
    VNAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_VENDOR_ATTR6 (
    PK INTEGER NOT NULL,
    VID GENERAL_NAME COLLATE ISO8859_1,
    VCLASS GENERAL_NAME COLLATE ISO8859_1,
    VNAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding statistics, DHCPv4 and DHCPv6 versions */

CREATE TABLE WSI_STATISTICS4 (
    PK INTEGER NOT NULL,
    PATH GENERAL_NAME COLLATE ISO8859_1,
    VAL AMOUNT_TYPE,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_STATISTICS6 (
    PK INTEGER NOT NULL,
    PATH GENERAL_NAME COLLATE ISO8859_1,
    VAL AMOUNT_TYPE,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding key/value pairs, DHCPv4 and DHCPv6 versions */

CREATE TABLE WSI_KEYVALUES4 (
    PK INTEGER NOT NULL,
    CLASS GENERAL_NAME COLLATE ISO8859_1,
    SUBCLASS GENERAL_NAME COLLATE ISO8859_1,
    KKEY GENERAL_NAME COLLATE ISO8859_1,
    KVALUE GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_KEYVALUES6 (
    PK INTEGER NOT NULL,
    CLASS GENERAL_NAME COLLATE ISO8859_1,
    SUBCLASS GENERAL_NAME COLLATE ISO8859_1,
    KKEY GENERAL_NAME COLLATE ISO8859_1,
    KVALUE GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding address bindings, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_ABINDINGS4 (
    PK INTEGER NOT NULL,
    CLIENT_ID HEXSTORE COLLATE ISO8859_1,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    OPTIONS BLOB sub_type 1 segment size 1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    BOOT_PROTO BOOT_PROTO_TYPE COLLATE ISO8859_1,
    LEASE_COMMIT TIME_T,
    LEASE_DURATION AMOUNT_TYPE,
    FIXED BOOL,
    TID_TYPE TID_CLASS,
    TID HEXSTORE COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_ABINDINGS6 (
    PK INTEGER NOT NULL,
    DUID HEXSTORE COLLATE ISO8859_1,
    IATYPE TYPE_AS_INT,
    IAID   ID_AS_INT,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    OPTIONS BLOB sub_type 1 segment size 1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    BOOT_PROTO BOOT_PROTO_TYPE COLLATE ISO8859_1,
    LEASE_COMMIT TIME_T,
    LEASE_DURATION AMOUNT_TYPE,
    FIXED BOOL,
    TID_TYPE TID_CLASS,
    TID HEXSTORE COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding network bindings for DHCPv6 */
CREATE TABLE DHCP_NBINDINGS6 (
    PK INTEGER NOT NULL,
    DUID HEXSTORE COLLATE ISO8859_1,
    IATYPE TYPE_AS_INT,
    IAID   ID_AS_INT,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    OPTIONS BLOB sub_type 1 segment size 1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    BOOT_PROTO BOOT_PROTO_TYPE COLLATE ISO8859_1,
    LEASE_COMMIT TIME_T,
    LEASE_DURATION AMOUNT_TYPE,
    FIXED BOOL,
    TID_TYPE TID_CLASS,
    TID HEXSTORE COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding network bindings for DHCPv4 - placeholder table only */
CREATE TABLE DHCP_NBINDINGS4 (
    PK INTEGER NOT NULL,
    CLIENT_ID HEXSTORE COLLATE ISO8859_1,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    OPTIONS BLOB sub_type 1 segment size 1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    BOOT_PROTO BOOT_PROTO_TYPE COLLATE ISO8859_1,
    LEASE_COMMIT TIME_T,
    LEASE_DURATION AMOUNT_TYPE,
    FIXED BOOL,
    TID_TYPE TID_CLASS,
    TID HEXSTORE COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* Private table of the failover module. Each record in this table refers to a DHCP_ABINDINGS4 record. */

CREATE TABLE DHCP_AFBINDINGS4 (
    PK INTEGER NOT NULL,
		IPADDR IP_ADDRESS COLLATE ISO8859_1,
		ACK_POTENTIAL_EXP_TIME TIME_T,
		LEASE_EXP_TIME TIME_T,
		UNACKED_UPDATE BOOL,
		SESSION_ID INT32_T,
		RCV_POTENTIAL_EXP_TIME TIME_T,
		SND_POTENTIAL_EXP_TIME TIME_T,
		STATE_START_TIME TIME_T,
		CLIENT_LAST_TRANS_TIME TIME_T,
		STATE GENERAL_NAME COLLATE ISO8859_1,
		UNDERLYINGS FOREIGN_KEY_ID,
		UNDERLYINGS_SC SELECT_CRITERIA COLLATE ISO8859_1);

/* Implements the 0..1 reference to the underlying binding record. */

CREATE TABLE DHCP_AFBINDINGS4_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);


/* A table for holding address pools (apools), DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_APOOLS4 (
    PK INTEGER NOT NULL,
    ENABLED BOOL,
    ALLOW EXPRESSION_TYPE COLLATE ISO8859_1,
    DENY EXPRESSION_TYPE COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    RELAY FOREIGN_KEY_ID,
    RELAY_SC SELECT_CRITERIA COLLATE ISO8859_1,
    PREFIX IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    PREF_LT AMOUNT_TYPE,
    VALID_LT AMOUNT_TYPE,
    RANGE FOREIGN_KEY_ID,
    RANGE_CL CLASS_NAME COLLATE ISO8859_1,
    XRANGE FOREIGN_KEY_ID,
    XRANGE_SC SELECT_CRITERIA COLLATE ISO8859_1,
    WEIGHT AMOUNT_TYPE,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_APOOLS4_EXC_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE DHCP_APOOLS4_RELAY_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE DHCP_APOOLS6 (
    PK INTEGER NOT NULL,
    ENABLED BOOL,
    ALLOW EXPRESSION_TYPE COLLATE ISO8859_1,
    DENY EXPRESSION_TYPE COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    NAME GENERAL_NAME  COLLATE ISO8859_1,
    RELAY FOREIGN_KEY_ID,
    RELAY_SC SELECT_CRITERIA COLLATE ISO8859_1,
    PREFIX IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    PREF_LT AMOUNT_TYPE,
    VALID_LT AMOUNT_TYPE,
    RANGE FOREIGN_KEY_ID,
    RANGE_CL CLASS_NAME COLLATE ISO8859_1,
    XRANGE FOREIGN_KEY_ID,
    XRANGE_SC SELECT_CRITERIA COLLATE ISO8859_1,
    WEIGHT AMOUNT_TYPE,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_APOOLS6_EXC_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE DHCP_APOOLS6_RELAY_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

/* A table for holding network pools (npools) for DHCPv6 */

CREATE TABLE DHCP_NPOOLS6 (
    PK INTEGER NOT NULL,
    ENABLED BOOL,
    ALLOW EXPRESSION_TYPE COLLATE ISO8859_1,
    DENY EXPRESSION_TYPE COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    RELAY FOREIGN_KEY_ID,
    RELAY_SC SELECT_CRITERIA COLLATE ISO8859_1,
    PREFIX IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    SUB_PREFIX_LEN AMOUNT_TYPE,
    PREF_LT AMOUNT_TYPE,
    VALID_LT AMOUNT_TYPE,
    XRANGE FOREIGN_KEY_ID,
    XRANGE_SC SELECT_CRITERIA COLLATE ISO8859_1,
    WEIGHT AMOUNT_TYPE,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_NPOOLS6_EXC_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE DHCP_NPOOLS6_RELAY_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

/* A table for holding network pools (npools) for DHCPv4 - placeholder table only */

CREATE TABLE DHCP_NPOOLS4 (
    PK INTEGER NOT NULL,
    ENABLED BOOL,
    ALLOW EXPRESSION_TYPE COLLATE ISO8859_1,
    DENY EXPRESSION_TYPE COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    RELAY FOREIGN_KEY_ID,
    RELAY_SC SELECT_CRITERIA COLLATE ISO8859_1,
    PREFIX IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    SUB_PREFIX_LEN AMOUNT_TYPE,
    PREF_LT AMOUNT_TYPE,
    VALID_LT AMOUNT_TYPE,
    XRANGE FOREIGN_KEY_ID,
    XRANGE_SC SELECT_CRITERIA COLLATE ISO8859_1,
    WEIGHT AMOUNT_TYPE,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_NPOOLS4_EXC_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE DHCP_NPOOLS4_RELAY_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

/* Tables for implementing security */

CREATE TABLE WSI_DOMAIN_ID (
    PK INTEGER NOT NULL,
    DOMAIN_ID DOM_ID_TYPE,
    SRC_PK INT32_T,
    SRC_TABLE GENERAL_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_DOMAIN_MSHIPS_LINKS (
    PK INTEGER NOT NULL,
    SRC_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    DEST_ATTR_NAME GENERAL_NAME COLLATE ISO8859_1,
    SRC_ATTR_REF FOREIGN_KEY_ID,
    DEST_ATTR_REF FOREIGN_KEY_ID,
    SRC_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_ATTR_REF_CL CLASS_NAME COLLATE ISO8859_1,
    DEST_NAME_KEY GENERAL_NAME COLLATE ISO8859_1,
    DEST_SEQ_KEY INDEX_TYPE);

CREATE TABLE WSI_DOMAIN_MSHIPS (
    PK INTEGER NOT NULL,
    DOM_LIST FOREIGN_KEY_ID,
    DOM_LIST_SC SELECT_CRITERIA COLLATE ISO8859_1);

CREATE TABLE WSI_SECURED_OBJECT (
    PK INTEGER NOT NULL,
    DOM_MEMBERSHIPS_REF FOREIGN_KEY_ID,
    DOM_MEMBERSHIPS_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for domains (device and user) */

CREATE TABLE WSI_DOMAINS (
    PK INTEGER NOT NULL,
    ID DOM_ID_TYPE,
    CLASS ID_CLASS_TYPE,
    PRIORITY AMOUNT_TYPE,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    DESCRIPTION LONG_TEXT,
    NAME_ID_REF FOREIGN_KEY_ID,
    NAME_ID_REF_CL CLASS_NAME COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE WSI_DOMAIN_NAME_ID (
    PK INTEGER NOT NULL,
    ID DOM_ID_TYPE,
    NAME GENERAL_NAME COLLATE ISO8859_1);

/* Tables for access IDs */

CREATE TABLE WSI_ACCESS_CONTROL (
    PK INTEGER NOT NULL,
    ACCESS_ID ACCESS_ID_TYPE,
    DOMAIN_ID DOM_ID_TYPE,
    RIGHTS RIGHTS_TYPE,
    DOMAIN_REF FOREIGN_KEY_ID,
    DOMAIN_REF_CL SELECT_CRITERIA COLLATE ISO8859_1,
    ACCOUNT_REF FOREIGN_KEY_ID,
    ACCOUNT_REF_CL SELECT_CRITERIA COLLATE ISO8859_1);

/* A table for accounts (device and user) */

CREATE TABLE WSI_ACCOUNTS (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    PASS GENERAL_NAME COLLATE ISO8859_1,
    ENABLED BOOL,
    DESCRIPTION LONG_TEXT,
    ID ACCESS_ID_TYPE,
    MOD_TIME TIME_T,
    CLASS ID_CLASS_TYPE,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding policies, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_POLICIES4 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    ENFORCE BOOL,
    DESCRIPTION LONG_TEXT,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_POLICIES6 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    ENFORCE BOOL,
    DESCRIPTION LONG_TEXT,
    OPTIONS BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding apool reclamation marks */

CREATE TABLE DHCP_APOOL_MARKS4 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    MARK IP_ADDRESS COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_APOOL_MARKS6 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    MARK IP_ADDRESS COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding npool reclamation marks for DHCPv6 */

CREATE TABLE DHCP_NPOOL_MARKS6 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    MARK IP_ADDRESS COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding npool reclamation marks for DHCPv4 - placeholder table only */

CREATE TABLE DHCP_NPOOL_MARKS4 (
    PK INTEGER NOT NULL,
    NAME GENERAL_NAME COLLATE ISO8859_1,
    MARK IP_ADDRESS COLLATE ISO8859_1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* A table for holding historical dhcp4 packets */

CREATE TABLE DHCP_PKTS4 (
    PK INTEGER NOT NULL,
    PKT_TYPE GENERAL_NAME COLLATE ISO8859_1,
    PRIMARY_ID HEXSTORE COLLATE ISO8859_1,
    SECONDARY_ID HEXSTORE COLLATE ISO8859_1,
    MOD_TIME TIME_T,
    PKT BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

CREATE TABLE DHCP_PKTS6 (
    PK INTEGER NOT NULL,
    PKT_TYPE GENERAL_NAME COLLATE ISO8859_1,
    PRIMARY_ID HEXSTORE COLLATE ISO8859_1,
    SECONDARY_ID HEXSTORE COLLATE ISO8859_1,
    MOD_TIME TIME_T,
    PKT BLOB sub_type 1 segment size 1,
    SECURED_REF FOREIGN_KEY_ID,
    SECURED_REF_CL CLASS_NAME COLLATE ISO8859_1);

/* Primary key definitions for all tables */
ALTER TABLE DHCP_ELEMENT_ATTR4 ADD CONSTRAINT PK_ELEMENT_ATTR4 PRIMARY KEY (PK);
ALTER TABLE DHCP_OPTION_ATTR4 ADD CONSTRAINT PK_OPTION_ATTR4 PRIMARY KEY (PK);
ALTER TABLE DHCP_ELEMENT_ATTR6 ADD CONSTRAINT PK_ELEMENT_ATTR6 PRIMARY KEY (PK);
ALTER TABLE DHCP_OPTION_ATTR6 ADD CONSTRAINT PK_OPTION_ATTR6 PRIMARY KEY (PK);
ALTER TABLE DHCP_VENDOR_ATTR4 ADD CONSTRAINT PK_VENDOR_ATTR4 PRIMARY KEY (PK);
ALTER TABLE DHCP_VENDOR_ATTR6 ADD CONSTRAINT PK_VENDOR_ATTR6 PRIMARY KEY (PK);
ALTER TABLE DHCP_ABINDINGS4 ADD CONSTRAINT PK_ABINDINGS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_ABINDINGS6 ADD CONSTRAINT PK_ABINDINGS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_AFBINDINGS4 ADD CONSTRAINT PK_AFBINDINGS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_ADDRESS_RANGE ADD CONSTRAINT PK_ADDRESS_RANGE PRIMARY KEY (PK);
ALTER TABLE DHCP_EXC_ADDR_RANGE ADD CONSTRAINT PK_EXC_ADDR_RANGE PRIMARY KEY (PK);
ALTER TABLE DHCP_NBINDINGS6 ADD CONSTRAINT PK_NBINDINGS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_NBINDINGS4 ADD CONSTRAINT PK_NBINDINGS4 PRIMARY KEY (PK);
ALTER TABLE WSI_ADDRESS ADD CONSTRAINT PK_ADDRESS PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS4 ADD CONSTRAINT PK_APOOLS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS4_EXC_LINKS ADD CONSTRAINT PK_APOOLS4_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS4_RELAY_LINKS ADD CONSTRAINT PK_APOOLS4_RELAY_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_AFBINDINGS4_LINKS ADD CONSTRAINT PK_AFBINDINGS4_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS6 ADD CONSTRAINT PK_APOOLS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS6_EXC_LINKS ADD CONSTRAINT PK_APOOLS6_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOLS6_RELAY_LINKS ADD CONSTRAINT PK_APOOLS6_RELAY_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS6 ADD CONSTRAINT PK_NPOOLS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS6_EXC_LINKS ADD CONSTRAINT PK_NPOOLS6_EXC_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS6_RELAY_LINKS ADD CONSTRAINT PK_NPOOLS6_RELAY_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS4 ADD CONSTRAINT PK_NPOOLS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS4_EXC_LINKS ADD CONSTRAINT PK_NPOOLS4_EXC_LINKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOLS4_RELAY_LINKS ADD CONSTRAINT PK_NPOOLS4_RELAY_LINKS PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT PK_DOMAINS PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_NAME_ID ADD CONSTRAINT PK_DOMAIN_NAME_ID PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_ID ADD CONSTRAINT PK_DOMAIN_ID PRIMARY KEY (PK);
ALTER TABLE WSI_ACCESS_CONTROL ADD CONSTRAINT PK_ACCESS_CONTROL PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_MSHIPS_LINKS ADD CONSTRAINT PK_DOMAIN_MSHIPS_LINKS PRIMARY KEY (PK);
ALTER TABLE WSI_DOMAIN_MSHIPS ADD CONSTRAINT PK_DOMAIN_MSHIPS PRIMARY KEY (PK);
ALTER TABLE WSI_ACCOUNTS ADD CONSTRAINT PK_ACCOUNTS PRIMARY KEY (PK);
ALTER TABLE WSI_SECURED_OBJECT ADD CONSTRAINT PK_SECURED_OBJECT PRIMARY KEY (PK);
ALTER TABLE DHCP_POLICIES4 ADD CONSTRAINT PK_POLICIES4 PRIMARY KEY (PK);
ALTER TABLE DHCP_POLICIES6 ADD CONSTRAINT PK_POLICIES6 PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOL_MARKS4 ADD CONSTRAINT PK_APOOL4_MARKS PRIMARY KEY (PK);
ALTER TABLE DHCP_APOOL_MARKS6 ADD CONSTRAINT PK_APOOL6_MARKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOL_MARKS6 ADD CONSTRAINT PK_NPOOL6_MARKS PRIMARY KEY (PK);
ALTER TABLE DHCP_NPOOL_MARKS4 ADD CONSTRAINT PK_NPOOL4_MARKS PRIMARY KEY (PK);
ALTER TABLE DHCP_PKTS4 ADD CONSTRAINT PK_PKTS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_PKTS6 ADD CONSTRAINT PK_PKTS6 PRIMARY KEY (PK);
ALTER TABLE WSI_KEYVALUES6 ADD CONSTRAINT PK_KEYVALUES6 PRIMARY KEY(PK);
ALTER TABLE WSI_KEYVALUES4 ADD CONSTRAINT PK_KEYVALUES4 PRIMARY KEY(PK);
ALTER TABLE WSI_STATISTICS4 ADD CONSTRAINT PK_STATISTICS4 PRIMARY KEY(PK);
ALTER TABLE WSI_STATISTICS6 ADD CONSTRAINT PK_STATISTICS6 PRIMARY KEY(PK);

/* DESCending indexes on primary keys for fast MAX search */
CREATE UNIQUE DESCENDING INDEX IDX_AA ON DHCP_ELEMENT_ATTR4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AB ON DHCP_OPTION_ATTR4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AC ON DHCP_ELEMENT_ATTR6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AD ON DHCP_OPTION_ATTR6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AE ON DHCP_VENDOR_ATTR4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AF ON DHCP_VENDOR_ATTR6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AG ON DHCP_ABINDINGS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AH ON DHCP_ABINDINGS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AI ON DHCP_AFBINDINGS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AJ ON DHCP_ADDRESS_RANGE(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AK ON DHCP_EXC_ADDR_RANGE(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AL ON DHCP_NBINDINGS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AM ON DHCP_NBINDINGS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AN ON WSI_ADDRESS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AO ON DHCP_APOOLS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AP ON DHCP_APOOLS4_EXC_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AQ ON DHCP_APOOLS4_RELAY_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AR ON DHCP_AFBINDINGS4_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AS ON DHCP_APOOLS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AT ON DHCP_APOOLS6_EXC_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AU ON DHCP_APOOLS6_RELAY_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AV ON DHCP_NPOOLS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AW ON DHCP_NPOOLS6_EXC_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AY ON DHCP_NPOOLS6_RELAY_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_AZ ON DHCP_NPOOLS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BA ON DHCP_NPOOLS4_EXC_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BB ON DHCP_NPOOLS4_RELAY_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BC ON WSI_DOMAINS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BD ON WSI_DOMAIN_NAME_ID(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BE ON WSI_DOMAIN_ID(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BF ON WSI_ACCESS_CONTROL(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BG ON WSI_DOMAIN_MSHIPS_LINKS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BH ON WSI_DOMAIN_MSHIPS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BI ON WSI_ACCOUNTS(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BJ ON WSI_SECURED_OBJECT(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BK ON DHCP_POLICIES4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BL ON DHCP_POLICIES6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BM ON DHCP_APOOL_MARKS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BN ON DHCP_APOOL_MARKS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BO ON DHCP_NPOOL_MARKS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BP ON DHCP_NPOOL_MARKS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BQ ON DHCP_PKTS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BR ON DHCP_PKTS6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BS ON WSI_KEYVALUES6(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BT ON WSI_KEYVALUES4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BU ON WSI_STATISTICS4(PK);
CREATE UNIQUE DESCENDING INDEX IDX_BV ON WSI_STATISTICS6(PK);

/* Foreign keys, indexes, etc. */
CREATE INDEX IDX_WSI_ADDRESS_SRC_PK ON WSI_ADDRESS(SRC_PK);
CREATE INDEX IDX_WSI_ADDRESS_ADDR ON WSI_ADDRESS(ADDR);

/* option attr4 */
ALTER TABLE DHCP_OPTION_ATTR4 ADD CONSTRAINT FK_OPTION_ATTR4 FOREIGN KEY (ELEMENT_ATTR_REF) REFERENCES DHCP_ELEMENT_ATTR4 (PK);
ALTER TABLE DHCP_OPTION_ATTR4 ADD CONSTRAINT FK_OPTION_ATTR4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_OPTION_ATTR4 ADD CONSTRAINT PK_OPTION_ATTR4_UNIQUE_VID UNIQUE (TAGPATH,VENDOR_ID);

/* option attr6 */
ALTER TABLE DHCP_OPTION_ATTR6 ADD CONSTRAINT FK_OPTION_ATTR6 FOREIGN KEY (ELEMENT_ATTR_REF) REFERENCES DHCP_ELEMENT_ATTR6 (PK);
ALTER TABLE DHCP_OPTION_ATTR6 ADD CONSTRAINT FK_OPTION_ATTR6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_OPTION_ATTR6 ADD CONSTRAINT PK_OPTION_ATTR6_UNIQUE_VID UNIQUE (TAGPATH,VENDOR_ID);

/* vendor attr4 */
ALTER TABLE DHCP_VENDOR_ATTR4 ADD CONSTRAINT FK_VENDOR_ATTR4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_VENDOR_ATTR4 ADD CONSTRAINT PK_VENDOR_ATTR4_UNIQUE_VCLASS UNIQUE (VCLASS);

/* vendor attr6 */
ALTER TABLE DHCP_VENDOR_ATTR6 ADD CONSTRAINT FK_VENDOR_ATTR6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_VENDOR_ATTR6 ADD CONSTRAINT PK_VENDOR_ATTR6_UNIQUE_VCLASS UNIQUE (VCLASS);

/* keyvalues4 */
ALTER TABLE WSI_KEYVALUES4 ADD CONSTRAINT FK_KEYVALUES4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_WSI_KEYVALUES4 ON WSI_KEYVALUES4 (KKEY,CLASS);

/* keyvalues6 */
ALTER TABLE WSI_KEYVALUES6 ADD CONSTRAINT FK_KEYVALUES6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_WSI_KEYVALUES6 ON WSI_KEYVALUES6 (KKEY,CLASS);

/* statistics4 */
ALTER TABLE WSI_STATISTICS4 ADD CONSTRAINT FK_STATISTICS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_WSI_STATISTICS4 ON WSI_STATISTICS4 (PATH);

/* statistics6 */
ALTER TABLE WSI_STATISTICS6 ADD CONSTRAINT FK_STATISTICS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_WSI_STATISTICS6 ON WSI_STATISTICS6 (PATH);

/* apools4 */
ALTER TABLE DHCP_APOOLS4_EXC_LINKS ADD CONSTRAINT FK_APOOLS4_EXC_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES DHCP_EXC_ADDR_RANGE (PK);
ALTER TABLE DHCP_APOOLS4_EXC_LINKS ADD CONSTRAINT FK_APOOLS4_EXC_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_APOOLS4 (PK);
ALTER TABLE DHCP_APOOLS4_RELAY_LINKS ADD CONSTRAINT FK_APOOLS4_RELAY_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_APOOLS4_RELAY_LINKS ADD CONSTRAINT FK_APOOLS4_RELAY_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_APOOLS4 (PK);
ALTER TABLE DHCP_APOOLS4 ADD CONSTRAINT FK_APOOLS4_RANGE FOREIGN KEY (RANGE) REFERENCES DHCP_ADDRESS_RANGE (PK);
ALTER TABLE DHCP_APOOLS4 ADD CONSTRAINT FK_APOOLS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE UNIQUE INDEX IDX_DHCP_APOOLS4_NAME ON DHCP_APOOLS4 (NAME);

/* apools6 */
ALTER TABLE DHCP_APOOLS6_EXC_LINKS ADD CONSTRAINT FK_APOOLS6_EXC_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES DHCP_EXC_ADDR_RANGE (PK);
ALTER TABLE DHCP_APOOLS6_EXC_LINKS ADD CONSTRAINT FK_APOOLS6_EXC_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_APOOLS6 (PK);
ALTER TABLE DHCP_APOOLS6_RELAY_LINKS ADD CONSTRAINT FK_APOOLS6_RELAY_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_APOOLS6_RELAY_LINKS ADD CONSTRAINT FK_APOOLS6_RELAY_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_APOOLS6 (PK);
ALTER TABLE DHCP_APOOLS6 ADD CONSTRAINT FK_APOOLS6_RANGE FOREIGN KEY (RANGE) REFERENCES DHCP_ADDRESS_RANGE (PK);
ALTER TABLE DHCP_APOOLS6 ADD CONSTRAINT FK_APOOLS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE UNIQUE INDEX IDX_DHCP_APOOLS6_NAME ON DHCP_APOOLS6 (NAME);

/* npools6 */
ALTER TABLE DHCP_NPOOLS6_EXC_LINKS ADD CONSTRAINT FK_NPOOLS6_EXC_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_NPOOLS6_EXC_LINKS ADD CONSTRAINT FK_NPOOLS6_EXC_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_NPOOLS6 (PK);
ALTER TABLE DHCP_NPOOLS6_RELAY_LINKS ADD CONSTRAINT FK_NPOOLS6_RELAY_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_NPOOLS6_RELAY_LINKS ADD CONSTRAINT FK_NPOOLS6_RELAY_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_NPOOLS6 (PK);
ALTER TABLE DHCP_NPOOLS6 ADD CONSTRAINT FK_NPOOLS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE UNIQUE INDEX IDX_DHCP_NPOOLS6_NAME ON DHCP_NPOOLS6 (NAME);

/* npools4 - placeholder table only */
ALTER TABLE DHCP_NPOOLS4_EXC_LINKS ADD CONSTRAINT FK_NPOOLS4_EXC_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_NPOOLS4_EXC_LINKS ADD CONSTRAINT FK_NPOOLS4_EXC_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_NPOOLS4 (PK);
ALTER TABLE DHCP_NPOOLS4_RELAY_LINKS ADD CONSTRAINT FK_NPOOLS4_RELAY_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_ADDRESS (PK);
ALTER TABLE DHCP_NPOOLS4_RELAY_LINKS ADD CONSTRAINT FK_NPOOLS4_RELAY_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_NPOOLS4 (PK);
ALTER TABLE DHCP_NPOOLS4 ADD CONSTRAINT FK_NPOOLS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);

/* domain memberships */
ALTER TABLE WSI_DOMAIN_MSHIPS_LINKS ADD CONSTRAINT FK_DOMAIN_MSHIPS_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES WSI_DOMAIN_MSHIPS (PK);
ALTER TABLE WSI_DOMAIN_MSHIPS_LINKS ADD CONSTRAINT FK_DOMAIN_MSHIPS_LINKS_DST FOREIGN KEY (DEST_ATTR_REF) REFERENCES WSI_DOMAIN_ID (PK);

/* secured objects */
ALTER TABLE WSI_SECURED_OBJECT ADD CONSTRAINT FK_SECURED_OBJECT FOREIGN KEY (DOM_MEMBERSHIPS_REF) REFERENCES WSI_DOMAIN_MSHIPS (PK);

/* address bindings4 */
ALTER TABLE DHCP_ABINDINGS4 ADD CONSTRAINT FK_ABINDINGS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_ABINDINGS4 ADD CONSTRAINT PK_ABINDINGS4_IPADDR  UNIQUE (IPADDR);
CREATE INDEX IDX_ABINDINGS4_BID ON DHCP_ABINDINGS4 (CLIENT_ID);
CREATE ASC INDEX IDX_ABINDINGS4_RELAY ON DHCP_ABINDINGS4 (RELAY,PK);
CREATE ASC INDEX IDX_ABINDINGS4_TID ON DHCP_ABINDINGS4 (TID_TYPE,TID);
CREATE ASC INDEX IDX_ABINDINGS4_BID_RELAY ON DHCP_ABINDINGS4 (CLIENT_ID, RELAY);

/* address bindings6 */
ALTER TABLE DHCP_ABINDINGS6 ADD CONSTRAINT FK_ABINDINGS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_ABINDINGS6 ADD CONSTRAINT PK_ABINDINGS6_IPADDR  UNIQUE (IPADDR);

CREATE INDEX IDX_ABINDINGS6_DUID ON DHCP_ABINDINGS6 (DUID);
CREATE INDEX IDX_ABINDINGS6_IAID ON DHCP_ABINDINGS6 (IAID);
CREATE ASC INDEX IDX_ABINDINGS6_RELAY ON DHCP_ABINDINGS6 (RELAY,PK);
CREATE ASC INDEX IDX_ABINDINGS6_TID ON DHCP_ABINDINGS6 (TID_TYPE,TID);
CREATE ASC INDEX IDX_ABINDINGS6_BID_RELAY ON DHCP_ABINDINGS6 (DUID, IAID, RELAY, IATYPE);

/* network bindings6 */
ALTER TABLE DHCP_NBINDINGS6 ADD CONSTRAINT FK_NBINDINGS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_NBINDINGS6 ADD CONSTRAINT PK_NBINDINGS6_IPADDR  UNIQUE (IPADDR);

CREATE INDEX IDX_NBINDINGS6_DUID ON DHCP_NBINDINGS6 (DUID);
CREATE INDEX IDX_NBINDINGS6_IAID ON DHCP_NBINDINGS6 (IAID);
CREATE ASC INDEX IDX_NBINDINGS6_RELAY ON DHCP_NBINDINGS6 (RELAY,PK);
CREATE ASC INDEX IDX_NBINDINGS6_TID ON DHCP_NBINDINGS6 (TID_TYPE,TID);
CREATE ASC INDEX IDX_NBINDINGS6_BID_RELAY ON DHCP_NBINDINGS6 (DUID, IAID, RELAY, IATYPE);

/* network bindings4 - placeholder table only */
ALTER TABLE DHCP_NBINDINGS4 ADD CONSTRAINT FK_NBINDINGS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_NBINDINGS4 ADD CONSTRAINT PK_NBINDINGS4_IPADDR  UNIQUE (IPADDR);

/* failover address bindings. Only for IP4. */
ALTER TABLE DHCP_AFBINDINGS4_LINKS ADD CONSTRAINT FK_AFBINDINGS4_LINKS_DEST FOREIGN KEY (DEST_ATTR_REF) REFERENCES DHCP_ABINDINGS4 (PK);
ALTER TABLE DHCP_AFBINDINGS4_LINKS ADD CONSTRAINT FK_AFBINDINGS4_LINKS_SRC FOREIGN KEY (SRC_ATTR_REF) REFERENCES DHCP_AFBINDINGS4 (PK);
ALTER TABLE DHCP_AFBINDINGS4 ADD CONSTRAINT PK_AFBINDINGS4_IPADDR  UNIQUE (IPADDR);

/* policies4 */
ALTER TABLE DHCP_POLICIES4 ADD CONSTRAINT FK_POLICIES4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_POLICIES4_NAME ON DHCP_POLICIES4 (NAME);

/* policies6 */
ALTER TABLE DHCP_POLICIES6 ADD CONSTRAINT FK_POLICIES6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_POLICIES6_NAME ON DHCP_POLICIES6 (NAME);

/* domains */
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT FK_DOMAINS_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT FK_DOMAIN_NAME_ID FOREIGN KEY (NAME_ID_REF) REFERENCES WSI_DOMAIN_NAME_ID (PK);
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT PK_DOMAINS_COMPOUND_UNIQUE UNIQUE (NAME,CLASS);
ALTER TABLE WSI_DOMAINS ADD CONSTRAINT PK_DOMAINS_UNIQUE_DOM_ID UNIQUE (ID);
CREATE INDEX IDX_DOMAINS_NAME ON WSI_DOMAINS (NAME);

/* domain ids */
CREATE ASC INDEX IDX_DOMAIN_ID_TABLE ON WSI_DOMAIN_ID (SRC_TABLE,PK);
CREATE ASC INDEX IDX_DOMAIN_ID_SRC ON WSI_DOMAIN_ID (SRC_PK);
CREATE ASC INDEX IDX_DOMAIN_ID_ID ON WSI_DOMAIN_ID (DOMAIN_ID,PK);
CREATE ASC INDEX IDX_WSI_DOMAIN_ID_T ON WSI_DOMAIN_ID (SRC_PK, DOMAIN_ID, SRC_TABLE);

/* domain name ids */
CREATE INDEX IDX_DOMAIN_NAME_ID_ID ON WSI_DOMAIN_NAME_ID (ID);

/* accounts */
ALTER TABLE WSI_ACCOUNTS ADD CONSTRAINT FK_ACCOUNTS_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE WSI_ACCOUNTS ADD CONSTRAINT PK_ACCOUNTS_COMPOUND_UNIQUE UNIQUE (NAME,CLASS);
ALTER TABLE WSI_ACCOUNTS ADD CONSTRAINT PK_ACCOUNTS_UNIQUE_ACCESS_ID UNIQUE (ID);

/* access controls */
ALTER TABLE WSI_ACCESS_CONTROL ADD CONSTRAINT PK_COMPOUND_UNIQUE_DOM_ID_AID UNIQUE (ACCESS_ID,DOMAIN_ID);
ALTER TABLE WSI_ACCESS_CONTROL ADD CONSTRAINT FK_ACCESS_CONTROL_ACCOUNTS FOREIGN KEY (ACCOUNT_REF) REFERENCES WSI_ACCOUNTS(PK);
ALTER TABLE WSI_ACCESS_CONTROL ADD CONSTRAINT FK_ACCESS_CONTROL_DOMAIN FOREIGN KEY (DOMAIN_REF) REFERENCES WSI_DOMAINS(PK);

/* historical dhcpv4 packets */
ALTER TABLE DHCP_PKTS4 ADD CONSTRAINT FK_PKTS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_PKTS4_1 ON DHCP_PKTS4 (PRIMARY_ID);
CREATE INDEX IDX_PKTS4_2 ON DHCP_PKTS4 (SECONDARY_ID);
CREATE INDEX IDX_PKTS4_3 ON DHCP_PKTS4 (PKT_TYPE);

/* historical dhcpv6 packets */
ALTER TABLE DHCP_PKTS6 ADD CONSTRAINT FK_PKTS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
CREATE INDEX IDX_PKTS6_1 ON DHCP_PKTS6 (PRIMARY_ID);
CREATE INDEX IDX_PKTS6_2 ON DHCP_PKTS6 (SECONDARY_ID);
CREATE INDEX IDX_PKTS6_3 ON DHCP_PKTS6 (PKT_TYPE);

/* reclaimer apool marks4 */
ALTER TABLE DHCP_APOOL_MARKS4 ADD CONSTRAINT FK_APOOL_MARKS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_APOOL_MARKS4 ADD CONSTRAINT PK_APOOL_MARKS4_UNIQUE_NAME UNIQUE (NAME);

/* reclaimer apool marks6 */
ALTER TABLE DHCP_APOOL_MARKS6 ADD CONSTRAINT FK_APOOL_MARKS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_APOOL_MARKS6 ADD CONSTRAINT PK_APOOL_MARKS6_UNIQUE_NAME UNIQUE (NAME);

/* reclaimer npool marks6 */
ALTER TABLE DHCP_NPOOL_MARKS6 ADD CONSTRAINT FK_NPOOL_MARKS6_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_NPOOL_MARKS6 ADD CONSTRAINT PK_NPOOL_MARKS6_UNIQUE_NAME UNIQUE (NAME);

/* reclaimer npool marks4 - placeholder table only */
ALTER TABLE DHCP_NPOOL_MARKS4 ADD CONSTRAINT FK_NPOOL_MARKS4_SECURED FOREIGN KEY (SECURED_REF) REFERENCES WSI_SECURED_OBJECT (PK);
ALTER TABLE DHCP_NPOOL_MARKS4 ADD CONSTRAINT PK_NPOOL_MARKS4_UNIQUE_NAME UNIQUE (NAME);
