/* A table for holding address pendings, DHCPv4 and DHCPv6 versions */

CREATE TABLE DHCP_APENDINGS4 (
    PK INTEGER NOT NULL,
    CLIENT_ID HEXSTORE COLLATE ISO8859_1,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    OFFER_TIME TIME_T);

CREATE TABLE DHCP_APENDINGS6 (
    PK INTEGER NOT NULL,
    DUID HEXSTORE COLLATE ISO8859_1,
    IATYPE TYPE_AS_INT,
    IAID   ID_AS_INT,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    OFFER_TIME TIME_T);

/* A table for holding network pendings for DHCPv6 */
CREATE TABLE DHCP_NPENDINGS6 (
    PK INTEGER NOT NULL,
    DUID HEXSTORE COLLATE ISO8859_1,
    IATYPE TYPE_AS_INT,
    IAID   ID_AS_INT,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    OFFER_TIME TIME_T);

/* A table for holding network pendings for DHCPv4 - placeholder table only */
CREATE TABLE DHCP_NPENDINGS4 (
    PK INTEGER NOT NULL,
    CLIENT_ID HEXSTORE COLLATE ISO8859_1,
    IPADDR IP_ADDRESS COLLATE ISO8859_1,
    PREFIX_LEN AMOUNT_TYPE,
    RELAY IP_ADDRESS COLLATE ISO8859_1,
    SRC_POOL GENERAL_NAME COLLATE ISO8859_1,
    OFFER_TIME TIME_T);

ALTER TABLE DHCP_APENDINGS4 ADD CONSTRAINT PK_APENDINGS4 PRIMARY KEY (PK);
ALTER TABLE DHCP_APENDINGS6 ADD CONSTRAINT PK_APENDINGS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_NPENDINGS6 ADD CONSTRAINT PK_NPENDINGS6 PRIMARY KEY (PK);
ALTER TABLE DHCP_NPENDINGS4 ADD CONSTRAINT PK_NPENDINGS4 PRIMARY KEY (PK);

/* address pendings4 */
ALTER TABLE DHCP_APENDINGS4 ADD CONSTRAINT PK_APENDINGS4_IPADDR  UNIQUE (IPADDR);
ALTER TABLE DHCP_APENDINGS4 ADD CONSTRAINT PK_APENDINGS4_BID_RELAY UNIQUE (CLIENT_ID,RELAY);
CREATE ASC INDEX IDX_APENDINGS4_BID_RELAY ON DHCP_APENDINGS4 (CLIENT_ID, RELAY);
CREATE ASC INDEX IDX_APENDINGS4_IPADDR ON DHCP_APENDINGS4 (IPADDR);

/* address pendings6 */
ALTER TABLE DHCP_APENDINGS6 ADD CONSTRAINT PK_APENDINGS6_IPADDR  UNIQUE (IPADDR);
ALTER TABLE DHCP_APENDINGS6 ADD CONSTRAINT PK_APENDINGS6_BID_RELAY UNIQUE (DUID,IAID,IATYPE,RELAY);
CREATE ASC INDEX IDX_APENDINGS6_BID_RELAY ON DHCP_APENDINGS6 (DUID, IAID, RELAY, IATYPE);
CREATE ASC INDEX IDX_APENDINGS6_IPADDR ON DHCP_APENDINGS6 (IPADDR);

/* network pendings6 */
ALTER TABLE DHCP_NPENDINGS6 ADD CONSTRAINT PK_NPENDINGS6_IPADDR  UNIQUE (IPADDR);
ALTER TABLE DHCP_NPENDINGS6 ADD CONSTRAINT PK_NPENDINGS6_BID_RELAY UNIQUE (DUID,IAID,IATYPE,RELAY);
CREATE ASC INDEX IDX_NPENDINGS6_BID_RELAY ON DHCP_NPENDINGS6 (DUID, IAID, RELAY, IATYPE);
CREATE ASC INDEX IDX_NPENDINGS6_IPADDR ON DHCP_NPENDINGS6 (IPADDR);

/* network pendings4 - placeholder table only */
ALTER TABLE DHCP_NPENDINGS4 ADD CONSTRAINT PK_NPENDINGS4_IPADDR  UNIQUE (IPADDR);

/* performance improvement for historical packet storage */
DROP INDEX IDX_PKTS4_3;
CREATE INDEX IDX_PKTS4_3 ON DHCP_PKTS4 (PRIMARY_ID,PKT_TYPE);

DROP INDEX IDX_PKTS6_3;
CREATE INDEX IDX_PKTS6_3 ON DHCP_PKTS6 (PRIMARY_ID,PKT_TYPE);
